package com.softslate.commerce.administrator.core;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.octo.captcha.module.config.CaptchaModuleConfigHelper;
import com.octo.captcha.module.struts.CaptchaServicePlugin;
import com.octo.captcha.service.CaptchaService;
import com.softslate.commerce.businessobjects.core.RecaptchaProcessor;

/**
 * @author David Tobey
 * 
 */
public class LoginForm extends BaseAdministratorForm {
	private static final long serialVersionUID = 3832906554818508340L;

	static Log log = LogFactory.getLog(LoginForm.class);

	private String userName = null;

	public String getUserName() {
		return (this.userName);
	}

	public void setUserName(String userName) {
		this.userName = userName;
	}

	private String password = null;

	public String getPassword() {
		return (this.password);
	}

	public void setPassword(String password) {
		this.password = password;
	}

	private String originalRequest = null;

	public String getOriginalRequest() {
		return (this.originalRequest);
	}

	public void setOriginalRequest(String originalRequest) {
		this.originalRequest = originalRequest;
	}

	private String textFromImage = null;

	public String getTextFromImage() {
		return textFromImage;
	}

	public void setTextFromImage(String textFromImage) {
		this.textFromImage = textFromImage;
	}

	private String ipAddress = null;

	public String getIpAddress() {
		return ipAddress;
	}

	public void setIpAddress(String ipAddress) {
		this.ipAddress = ipAddress;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isDebugEnabled())
			log.debug("Starting validation.");
		initializeProperties(mapping, request);
		setIpAddress(getRequest().getRemoteAddr());
		if (getUserName() == null || getUserName().length() == 0) {
			String message = getMessage("administrator/administrator",
					"administrator.userName", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}
		if (getPassword() == null || getPassword().length() == 0) {
			String message = getMessage("administrator/administrator",
					"administrator.password", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}

		// Validate CAPTCHA
		try {
			String useLegacyCaptchaMethod = (String) getSettings().getValue("useLegacyCaptchaMethod");
			String useImageVerificationWithAdminLogins = (String) getSettings().getValue(
					"useImageVerificationWithAdminLogins");
			if (getErrors().isEmpty() && useImageVerificationWithAdminLogins != null
					&& useImageVerificationWithAdminLogins.equals("1") && useLegacyCaptchaMethod != null
					&& useLegacyCaptchaMethod.equals("1")) {

				// Legacy JCaptcha method
				if (getTextFromImage() != null) {
					setTextFromImage(getTextFromImage().toLowerCase());
				}
				CaptchaService service = CaptchaServicePlugin.getInstance().getService();
				String captchaID = CaptchaModuleConfigHelper.getId(request);
				if (captchaID == null || getTextFromImage() == null || getTextFromImage().length() == 0
						|| !service.validateResponseForID(captchaID, getTextFromImage()).booleanValue()) {
					setTextFromImage(null);
					getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.invalidCaptcha"));
				}
			} else if (getErrors().isEmpty() && useImageVerificationWithAdminLogins != null
					&& useImageVerificationWithAdminLogins.equals("1")) {

				// Google Recaptcha method
				RecaptchaProcessor rp = getBusinessObjectFactory().createObject(RecaptchaProcessor.class);
				Map<String, String> params = new HashMap();
				params.put("response", request.getParameter("g-recaptcha-response"));
				Map response = rp.verifyRecaptcha(params);
				if (response == null || response.get("success") == null
						|| response.get("success").equals(Boolean.FALSE)) {
					getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.invalidReCaptcha"));
				}
			}

		} catch (Exception e) {
			log.error(e.toString(), e);
			setTextFromImage(null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.invalidReCaptcha"));
		}
		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		return getErrors();
	}
}
