package com.softslate.commerce.administrator.email;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.email.Template;
import com.softslate.commerce.businessobjects.email.TemplateProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * "/administrator/TemplateEditForm.do". Loads in the current values of the
 * record being edited and populates a <code>TemplateAddEditForm</code>
 * object.
 */
public class TemplateEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(TemplateAddFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) throws Exception {
		if (log.isDebugEnabled()) log.debug("Starting action.");
		TemplateAddEditForm baseForm = (TemplateAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Get the record to edit
		TemplateProcessor processor =  baseForm.getBusinessObjectFactory().createObject(TemplateProcessor.class);
		Template template = processor.loadTemplateFromID(PropertyUtils.describe(form));
		BeanUtils.copyProperties(baseForm, template);
		
		
		// Enforce role permissions
		if (!request.isUserInRole("superuser") && !request.isUserInRole("administrator") && !request.isUserInRole("templateEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home", "Templates", "Template Details" });
		request.setAttribute("breadcrumbs", bcs);

		request.setAttribute("template", template);
		request.setAttribute("templateAddEditForm", baseForm);
		return mapping.findForward("success");
	}
}
