package com.softslate.commerce.administrator.order;

import java.util.Arrays;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.administrator.core.BaseSerializableForm;
import com.softslate.commerce.businessobjects.order.Country;

/**
 * Struts form class corresponding to requests to "/administrator/Country.do".
 * Provides properties to control searching, viewing, editing, and deleting
 * records in the <code>sscCountry</code> table.
 * 
 * @author David Tobey
 * 
 */
public class CountryForm extends BaseSerializableForm {
    private static final long serialVersionUID = 4049917151823475766L;

    static Log log = LogFactory.getLog(CountryForm.class);

    private String idField = "countryID";

    private String nameField = "name";

    private String childName = "States";

    private String entityProperty = "country";

    private Class entity = Country.class;

    private String[] searchableIntFields = { "countryID" };

    private String[] searchableFields = { "code", "name" };

    private String[] fields = { "name", "code", "countryOrder",
            "allowOtherStates", "isActive" };

    private String[] fieldLables = { "Name", "Code", "Order", "'Other' States",
            "Active" };

    private String[] displayFields = { "name", "code", "countryOrder",
            "allowOtherStates", "isActive" };

    private String sortByString = "countryOrder";

    private String sortAscOrDesc = "ASC";

    private String formAction = "/Country.do";

    private String addFormAction = "/CountryAddForm.do";

    private String editFormAction = "/CountryEditForm.do";

    private String editDeleteFormAction = "/CountryEditDelete.do";

    private String[] countryID = null;

    private String[] code = null;

    private String[] name = null;

    private String[] countryOrder = null;

    private String[] allowOtherStates = null;

    private String[] isActive = null;

    public String getSortAscOrDesc() {
        return sortAscOrDesc;
    }

    public void setSortAscOrDesc(String sortAscOrDesc) {
        this.sortAscOrDesc = sortAscOrDesc;
    }

    public String[] getAllowOtherStates() {
        return allowOtherStates;
    }

    public void setAllowOtherStates(String[] allowOtherStates) {
        this.allowOtherStates = allowOtherStates;
    }

    public String[] getCode() {
        return code;
    }

    public void setCode(String[] code) {
        this.code = code;
    }

    public String[] getCountryOrder() {
        return countryOrder;
    }

    public void setCountryOrder(String[] countryOrder) {
        this.countryOrder = countryOrder;
    }

    public String[] getIsActive() {
        return isActive;
    }

    public void setIsActive(String[] isActive) {
        this.isActive = isActive;
    }

    public String[] getName() {
        return name;
    }

    public void setName(String[] name) {
        this.name = name;
    }

    public String getIdField() {
        return idField;
    }

    public void setIdField(String idField) {
        this.idField = idField;
    }

    public String[] getFields() {
        return fields;
    }

    public void setFields(String[] fields) {
        this.fields = fields;
    }

    public String[] getFieldLables() {
        return fieldLables;
    }

    public void setFieldLables(String[] fieldLables) {
        this.fieldLables = fieldLables;
    }

    public String[] getDisplayFields() {
        return displayFields;
    }

    public void setDisplayFields(String[] displayFields) {
        this.displayFields = displayFields;
    }

    public String getSortByString() {
        return sortByString;
    }

    public void setSortByString(String sortByString) {
        this.sortByString = sortByString;
    }

    public String getFormAction() {
        return formAction;
    }

    public void setFormAction(String formAction) {
        this.formAction = formAction;
    }

    public String getAddFormAction() {
        return addFormAction;
    }

    public void setAddFormAction(String addFormAction) {
        this.addFormAction = addFormAction;
    }

    public String getEditFormAction() {
        return editFormAction;
    }

    public void setEditFormAction(String editFormAction) {
        this.editFormAction = editFormAction;
    }

    public String getEditDeleteFormAction() {
        return editDeleteFormAction;
    }

    public void setEditDeleteFormAction(String editDeleteFormAction) {
        this.editDeleteFormAction = editDeleteFormAction;
    }

    public String[] getCountryID() {
        return countryID;
    }

    public void setCountryID(String[] countryID) {
        this.countryID = countryID;
    }

    public String getChildName() {
        return childName;
    }

    public void setChildName(String childName) {
        this.childName = childName;
    }

    public Class getEntity() {
        return entity;
    }

    public void setEntity(Class entity) {
        this.entity = entity;
    }

    public String getEntityProperty() {
        return entityProperty;
    }

    public void setEntityProperty(String entityProperty) {
        this.entityProperty = entityProperty;
    }

    public String getNameField() {
        return nameField;
    }

    public void setNameField(String nameField) {
        this.nameField = nameField;
    }

    public String[] getSearchableFields() {
        return searchableFields;
    }

    public void setSearchableFields(String[] searchableFields) {
        this.searchableFields = searchableFields;
    }

    public String[] getSearchableIntFields() {
        return searchableIntFields;
    }

    public void setSearchableIntFields(String[] searchableIntFields) {
        this.searchableIntFields = searchableIntFields;
    }

    public ActionErrors validate(ActionMapping mapping,
            HttpServletRequest request) {
        if (log.isDebugEnabled())
            log.debug("Starting validation.");
        setProperties(request);
        ActionErrors errors = new ActionErrors();
        if (getCountryID() != null) {
            for (int i = 0; i < getCountryID().length; i++) {
                if (Arrays.asList(getDisplayFields()).contains("code")) {
                    if (getCode().length < i + 1 || getCode()[i] == null
                            || getCode()[i].equals("")) {
                        errors.add(ActionMessages.GLOBAL_MESSAGE,
                                new ActionMessage("errors.required", "Code"));
                    }
                }
                if (Arrays.asList(getDisplayFields()).contains("name")) {
                    if (getName().length < i + 1 || getName()[i] == null
                            || getName()[i].equals("")) {
                        errors.add(ActionMessages.GLOBAL_MESSAGE,
                                new ActionMessage("errors.required", "Name"));
                    }
                }
                if (Arrays.asList(getDisplayFields()).contains("countryOrder")) {
                    if (getCountryOrder().length < i + 1
                            || getCountryOrder()[i] == null
                            || getCountryOrder()[i].equals("")) {
                        errors.add(ActionMessages.GLOBAL_MESSAGE,
                                new ActionMessage("errors.required",
                                        "Country Order"));
                    }
                }
                if (Arrays.asList(getDisplayFields()).contains(
                        "allowOtherStates")) {
                    if (getAllowOtherStates().length < i + 1
                            || getAllowOtherStates()[i] == null
                            || getAllowOtherStates()[i].equals("")) {
                        errors.add(ActionMessages.GLOBAL_MESSAGE,
                                new ActionMessage("errors.required",
                                        "Allow Other States"));
                    }
                }
                if (Arrays.asList(getDisplayFields()).contains("isActive")) {
                    if (getIsActive().length < i + 1
                            || getIsActive()[i] == null
                            || getIsActive()[i].equals("")) {
                        errors.add(ActionMessages.GLOBAL_MESSAGE,
                                new ActionMessage("errors.required",
                                        "Is Active"));
                    }
                }
            }
        }
        return errors;
    }
}
