package com.softslate.commerce.administrator.product;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.Attribute;
import com.softslate.commerce.businessobjects.product.AttributeProcessor;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * <code>/administrator/OptionAddForm.do</code>. Creates an empty
 * <code>OptionAddEditForm</code> object.
 * 
 * @author David Tobey
 * 
 */
public class OptionAddFormAction extends BaseAction {

	static Log log = LogFactory.getLog(OptionAddFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		OptionAddEditForm baseForm = (OptionAddEditForm) form;

		if (request.getAttribute("reset") != null
				&& request.getAttribute("reset").equals("true")) {
			Class classDefinition = baseForm.getClass();
			baseForm = (OptionAddEditForm) classDefinition.newInstance();
		}

		baseForm.initializeProperties(mapping, request);
		request.setAttribute("optionAddEditForm", baseForm);
		baseForm.setAttributeID(request.getParameter("attributeID"));
		baseForm.setProductID(request.getParameter("productID"));
		baseForm.setParentName(request.getParameter("parentName"));

		// Load the parent and grandparent if an id is present
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		Attribute attribute = null;
		if (log.isDebugEnabled())
			log.debug("Checking for parent: " + baseForm.getAttributeID());
		if (baseForm.getAttributeID() != null
				&& baseForm.getAttributeID().length() > 0) {
			AttributeProcessor parentProcessor =  bof
					.createObject(AttributeProcessor.class);
			attribute = parentProcessor.loadAttributeFromID(PropertyUtils
					.describe(baseForm));
			request.setAttribute("attribute", attribute);
		}
		Product product = null;
		if (log.isDebugEnabled())
			log.debug("Checking for parent: " + baseForm.getProductID());
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {
			ProductProcessor gpProcessor =  bof
					.createObject(ProductProcessor.class);
			product = gpProcessor.loadProductFromID(PropertyUtils
					.describe(baseForm));
			request.setAttribute("product", product);
		}

		// Navigation
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Products",
					"Product Details", "Attributes", "Attribute Details",
					"Options", "Add Option" });
			request.setAttribute("breadcrumbs", bcs);
			
			// Create childSubNavigation collection
			Collection childsn = Arrays.asList(new String[] {
					"Attribute Details", "Options", "Add Option" });
			request.setAttribute("childSubNavigation", childsn);

		} else {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Attributes",
					"Attribute Details", "Options", "Add Option" });
			request.setAttribute("breadcrumbs", bcs);
		}

		return mapping.findForward("success");
	}
}
