package com.softslate.commerce.administrator.product;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.product.ProductInitializer;

/**
 * Struts action class that corresponds to requests to
 * "/administrator/ProductEditDelete.do". Processes a request to delete and edit
 * multiple records in the system. These requests are generated when the
 * adminstrator is deleting or editing multiple records simultaneously while in
 * edit mode.
 * 
 * @author David Tobey
 * @version $Id: ProductEditDeleteAction.java,v 1.1 2005/02/25 19:35:46 dtobey
 *          Exp $
 */
public class ProductEditDeleteAction extends BaseAction {

	static Log log = LogFactory.getLog(ProductEditDeleteAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		ProductForm baseForm = (ProductForm) form;

		// Prepare parameters for the edit/delete processing, and process
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		ProductProcessor processor =  bof
				.createObject(ProductProcessor.class);
		Map parameters = PropertyUtils.describe(form);
		for (int j = 0; j < baseForm.getDisplayFields().length; j++) {
			String displayField = baseForm.getDisplayFields()[j];

			// Bug where the primaryCategoryID field gets renamed to
			// primaryCategory
			if (displayField.equals("primaryCategory")) {
				displayField = "primaryCategoryID";
				baseForm.getDisplayFields()[j] = "primaryCategoryID";
			}

			// Handle setting the seoCodes as null if they are empty
			if (displayField.equals("seoCode") && baseForm.getSeoCode() != null) {
				for (int k = 0; k < baseForm.getSeoCode().length; k++) {
					if (baseForm.getSeoCode()[k].equals("")) {
						baseForm.getSeoCode()[k] = null;
					}
				}
			}

			parameters.put(displayField, PropertyUtils.getProperty(baseForm,
					displayField));
		}
		if (baseForm.getParentIDField().equals("parentProductID")) {
			parameters.put("parentProductID", PropertyUtils.getProperty(
					baseForm, "productID"));
		} else {
			parameters.put(baseForm.getIdField(), PropertyUtils.getProperty(
					baseForm, baseForm.getIdField()));
		}
		parameters.put("deleteID", baseForm.getDeleteID());
		processor.editDeleteProducts(parameters);

		// Reload builtin categories in the application scope.
		if (baseForm.getParentIDField().equals("categoryID")) {
			ProductInitializer pi = new ProductInitializer();
			pi.setServlet(servlet);
			pi.initializeBuiltInCategories(bof);
		}

		// Set the success message
		ActionMessages messages = new ActionMessages();
		messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
				"messages.recordsSaved"));
		saveMessages(request, messages);

		return mapping.findForward("success");
	}
}
