package com.softslate.commerce.businessobjects.customer;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing a customer within the system. <code>CustomerBean</code>
 * is the default implementation of the <code>Customer</code> interface for
 * the application.
 * <p>
 * The database stores customers in the <code>sscCustomer</code> table, and
 * this interface can be used to represent a single row of that table. The
 * Hibernate configuration file <code>CustomerBean.hbm.xml</code> maps that
 * table's columns to the properties in this object.
 * 
 * @author David Tobey
 * 
 * @see com.softslate.commerce.businessobjects.customer.Customer
 */
@Audited
@AuditTable("AudCustomer")
public class CustomerBean extends BaseBusinessObject implements Customer,
		Serializable {
	private static final long serialVersionUID = 3906372644591645747L;

	static Log log = LogFactory.getLog(CustomerBean.class);

	private int customerID = 0;

	public int getCustomerID() {
		return (this.customerID);
	}

	public void setCustomerID(int customerID) {
		this.customerID = customerID;
	}

	private String userName = null;

	public String getUserName() {
		return (this.userName);
	}

	public void setUserName(String userName) {
		this.userName = userName;
	}

	private String decryptedPassword = null;

	public String getDecryptedPassword() {
		return (this.decryptedPassword);
	}

	public void setDecryptedPassword(String decryptedPassword) {
		this.decryptedPassword = decryptedPassword;
	}

	private String password = null;

	public String getPassword() {
		return (this.password);
	}

	public void setPassword(String password) {
		this.password = password;
	}

	private String created = null;

	public String getCreated() {
		return (this.created);
	}

	public void setCreated(String created) {
		this.created = created;
	}

	private String lastLogin = null;

	public String getLastLogin() {
		return (this.lastLogin);
	}

	public void setLastLogin(String lastLogin) {
		this.lastLogin = lastLogin;
	}

	private String formattedCreated = null;

	public String getFormattedCreated() {
		return (this.formattedCreated);
	}

	public void setFormattedCreated(String formattedCreated) {
		this.formattedCreated = formattedCreated;
	}

	private String formattedLastLogin = null;

	public String getFormattedLastLogin() {
		return (this.formattedLastLogin);
	}

	public void setFormattedLastLogin(String formattedLastLogin) {
		this.formattedLastLogin = formattedLastLogin;
	}

	private String status = null;

	public String getStatus() {
		return (this.status);
	}

	public void setStatus(String status) {
		this.status = status;
	}

	private String firstName = null;

	public String getFirstName() {
		return (this.firstName);
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	private String lastName = null;

	public String getLastName() {
		return (this.lastName);
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	private String organization = null;

	public String getOrganization() {
		return (this.organization);
	}

	public void setOrganization(String organization) {
		this.organization = organization;
	}

	private String address1 = null;

	public String getAddress1() {
		return (this.address1);
	}

	public void setAddress1(String address1) {
		this.address1 = address1;
	}

	private String address2 = null;

	public String getAddress2() {
		return (this.address2);
	}

	public void setAddress2(String address2) {
		this.address2 = address2;
	}

	private String city = null;

	public String getCity() {
		return (this.city);
	}

	public void setCity(String city) {
		this.city = city;
	}

	private String state = null;

	public String getState() {
		return (this.state);
	}

	public void setState(String state) {
		this.state = state;
	}

	private String country = null;

	public String getCountry() {
		return (this.country);
	}

	public void setCountry(String country) {
		this.country = country;
	}

	private String postalCode = null;

	public String getPostalCode() {
		return (this.postalCode);
	}

	public void setPostalCode(String postalCode) {
		this.postalCode = postalCode;
	}

	private String phone1 = null;

	public String getPhone1() {
		return (this.phone1);
	}

	public void setPhone1(String phone1) {
		this.phone1 = phone1;
	}

	private String phone2 = null;

	public String getPhone2() {
		return (this.phone2);
	}

	public void setPhone2(String phone2) {
		this.phone2 = phone2;
	}

	private String email1 = null;

	public String getEmail1() {
		return (this.email1);
	}

	public void setEmail1(String email1) {
		this.email1 = email1;
	}

	private String email2 = null;

	public String getEmail2() {
		return (this.email2);
	}

	public void setEmail2(String email2) {
		this.email2 = email2;
	}

	private String extra1 = null;

	public String getExtra1() {
		return (this.extra1);
	}

	public void setExtra1(String extra1) {
		this.extra1 = extra1;
	}

	private String extra2 = null;

	private String extra3 = null;

	private String extra4 = null;

	private String extra5 = null;

	public String getExtra2() {
		return (this.extra2);
	}

	public void setExtra2(String extra2) {
		this.extra2 = extra2;
	}

	public String getExtra3() {
		return extra3;
	}

	public void setExtra3(String extra3) {
		this.extra3 = extra3;
	}

	public String getExtra4() {
		return extra4;
	}

	public void setExtra4(String extra4) {
		this.extra4 = extra4;
	}

	public String getExtra5() {
		return extra5;
	}

	public void setExtra5(String extra5) {
		this.extra5 = extra5;
	}

	private String notes = null;

	public String getNotes() {
		return (this.notes);
	}

	public void setNotes(String notes) {
		this.notes = notes;
	}
	
	private String autoLoginToken = null;

	public String getAutoLoginToken() {
		return autoLoginToken;
	}

	public void setAutoLoginToken(String autoLoginToken) {
		this.autoLoginToken = autoLoginToken;
	}

	private Collection customerAddressCollection = null;

	public Collection getCustomerAddressCollection() {
		return (this.customerAddressCollection);
	}

	public void setCustomerAddressCollection(
			Collection customerAddressCollection) {
		this.customerAddressCollection = customerAddressCollection;
	}

	// Aliases for the above getters and setters...

	public String getBillingFirstName() {
		return (getFirstName());
	}

	public void setBillingFirstName(String billingFirstName) {
		setFirstName(billingFirstName);
	}

	public String getBillingLastName() {
		return (getLastName());
	}

	public void setBillingLastName(String billingLastName) {
		setLastName(billingLastName);
	}

	public String getBillingOrganization() {
		return (getOrganization());
	}

	public void setBillingOrganization(String billingOrganization) {
		setOrganization(billingOrganization);
	}

	public String getBillingAddress1() {
		return (getAddress1());
	}

	public void setBillingAddress1(String billingAddress1) {
		setAddress1(billingAddress1);
	}

	public String getBillingAddress2() {
		return (getAddress2());
	}

	public void setBillingAddress2(String billingAddress2) {
		setAddress2(billingAddress2);
	}

	public String getBillingCity() {
		return (getCity());
	}

	public void setBillingCity(String billingCity) {
		setCity(billingCity);
	}

	public String getBillingState() {
		return (getState());
	}

	public void setBillingState(String billingState) {
		setState(billingState);
	}

	public String getBillingCountry() {
		return (getCountry());
	}

	public void setBillingCountry(String billingCountry) {
		setCountry(billingCountry);
	}

	public String getBillingPostalCode() {
		return (getPostalCode());
	}

	public void setBillingPostalCode(String billingPostalCode) {
		setPostalCode(billingPostalCode);
	}

	public String getBillingPhone1() {
		return (getPhone1());
	}

	public void setBillingPhone1(String billingPhone1) {
		setPhone1(billingPhone1);
	}

	public String getBillingPhone2() {
		return (getPhone2());
	}

	public void setBillingPhone2(String billingPhone2) {
		setPhone2(billingPhone2);
	}

	public String getBillingEmail1() {
		return (getEmail1());
	}

	public void setBillingEmail1(String billingEmail1) {
		setEmail1(billingEmail1);
	}

	public String getBillingEmail2() {
		return (getEmail2());
	}

	public void setBillingEmail2(String billingEmail2) {
		setEmail2(billingEmail2);
	}

	public String getBillingExtra1() {
		return (getExtra1());
	}

	public void setBillingExtra1(String billingExtra1) {
		setExtra1(billingExtra1);
	}

	public String getBillingExtra2() {
		return (getExtra2());
	}

	public void setBillingExtra2(String billingExtra2) {
		setExtra2(billingExtra2);
	}

	public String getBillingExtra3() {
		return (getExtra3());
	}

	public void setBillingExtra3(String billingExtra3) {
		setExtra3(billingExtra3);
	}

	public String getBillingExtra4() {
		return (getExtra4());
	}

	public void setBillingExtra4(String billingExtra4) {
		setExtra4(billingExtra4);
	}

	public String getBillingExtra5() {
		return (getExtra5());
	}

	public void setBillingExtra5(String billingExtra5) {
		setExtra5(billingExtra5);
	}

	public String getBillingNotes() {
		return (getNotes());
	}

	public void setBillingNotes(String billingNotes) {
		setNotes(billingNotes);
	}

	public CustomerAddress getFirstAddress() {
		if (getCustomerAddressCollection() == null) {
			return null;
		}
		Iterator iterator1 = getCustomerAddressCollection().iterator();
		if (iterator1.hasNext()) {
			return (CustomerAddress) iterator1.next();
		} else {
			return null;
		}
	}

	private Collection orderHistory = new ArrayList();

	public void setOrderHistory(Collection orderHistory) {
		this.orderHistory = orderHistory;
	}

	public Collection getOrderHistory() {
		return orderHistory;
	}

	private Collection orders = null;

	@NotAudited
	public Collection getOrders() {
		return orders;
	}

	public void setOrders(Collection orders) {
		this.orders = orders;
	}

	private Collection customerWishLists = null;

	public Collection getCustomerWishLists() {
		return (this.customerWishLists);
	}

	public void setCustomerWishLists(Collection customerWishLists) {
		this.customerWishLists = customerWishLists;
	}

	private String cimProfileID = null;

	public String getCimProfileID() {
		return cimProfileID;
	}

	public void setCimProfileID(String cimProfileID) {
		this.cimProfileID = cimProfileID;
	}

	private boolean canUsePO = false;

	public boolean getCanUsePO() {
		return canUsePO;
	}

	public void setCanUsePO(boolean canUsePO) {
		this.canUsePO = canUsePO;
	}

	private String passwordToken = null;

	public String getPasswordToken() {
		return passwordToken;
	}

	public void setPasswordToken(String passwordToken) {
		this.passwordToken = passwordToken;
	}

	private String passwordTokenDate = null;

	public String getPasswordTokenDate() {
		return passwordTokenDate;
	}

	public void setPasswordTokenDate(String passwordTokenDate) {
		this.passwordTokenDate = passwordTokenDate;
	}
}
