package com.softslate.commerce.businessobjects.order;

import java.util.Collection;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface representing a country within the system.
 * <p>
 * When the application needs to create an instance that implements
 * <code>sscCountry</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the "countryImplementer"
 * setting in the <code>appComponents.properties</code> file.
 * <p>
 * The default "countryImplementer" is
 * {@link com.softslate.commerce.businessobjects.order.CountryBean}.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudCountry")
public interface Country extends BusinessObject {
	abstract boolean getAllowOtherStates();

	abstract void setAllowOtherStates(boolean allowOtherStates);

	abstract String getCode();

	abstract void setCode(String code);

	abstract int getCountryOrder();

	abstract void setCountryOrder(int countryOrder);

	abstract boolean getIsActive();

	abstract void setIsActive(boolean isActive);

	abstract String getName();

	abstract void setName(String name);

	abstract int getCountryID();

	abstract void setCountryID(int countryID);

	abstract Collection getStates();

	abstract void setStates(Collection states);
}
