package com.softslate.commerce.businessobjects.order;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;
import com.softslate.commerce.businessobjects.product.Attribute;
import com.softslate.commerce.businessobjects.product.Option;

/**
 * Interface representing a order item attribute within the system. An order
 * item attribute corresponds to a single
 * {@link com.softslate.commerce.businessobjects.product.Attribute} of a product
 * that a customer purchases in an order. The <code>attributeValue</code>
 * property corresponds the value the customer selected for the attribute.
 * <p>
 * When the application needs to create an instance that implements
 * <code>sscOrderItemAttribute</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "orderItemAttributeImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "orderItemAttributeImplementer" is
 * {@link com.softslate.commerce.businessobjects.order.OrderItemAttributeBean}.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudOrderItemAttribute")
public interface OrderItemAttribute extends BusinessObject {
	abstract int getOrderItemAttributeID();

	abstract void setOrderItemAttributeID(int orderItemAttributeID);

	abstract Double getAttributeAltPrice();

	abstract void setAttributeAltPrice(Double attributeAltPrice);

	abstract String getAttributeCode();

	abstract void setAttributeCode(String attributeCode);

	abstract int getAttributeID();

	abstract void setAttributeID(int attributeID);

	abstract String getAttributeName();

	abstract void setAttributeName(String attributeName);

	abstract Double getAttributeUnitCost();

	abstract void setAttributeUnitCost(Double attributeUnitCost);

	abstract Double getAttributeUnitPrice();

	abstract void setAttributeUnitPrice(Double attributeUnitPrice);

	abstract String getAttributeValue();

	abstract void setAttributeValue(String attributeValue);

	abstract Double getAttributeWeight();

	abstract void setAttributeWeight(Double attributeWeight);

	abstract Double getOptionAltPrice();

	abstract void setOptionAltPrice(Double optionAltPrice);

	abstract String getOptionCode();

	abstract void setOptionCode(String optionCode);

	abstract int getOptionID();

	abstract void setOptionID(int optionID);

	abstract String getOptionName();

	abstract void setOptionName(String optionName);

	abstract Double getOptionUnitCost();

	abstract void setOptionUnitCost(Double optionUnitCost);

	abstract Double getOptionUnitPrice();

	abstract void setOptionUnitPrice(Double optionUnitPrice);

	abstract Double getOptionWeight();

	abstract void setOptionWeight(Double optionWeight);

	abstract int getOrderItemID();

	abstract void setOrderItemID(int orderItemID);

	abstract Double getTotal();

	abstract void setTotal(Double total);

	abstract Double getWeight();

	abstract void setWeight(Double total);
	
	abstract Double getCostTotal();

	abstract void setCostTotal(Double costTotal);

	abstract OrderItem getOrderItem();

	abstract void setOrderItem(OrderItem orderItem);

	abstract Attribute getAttribute();

	abstract void setAttribute(Attribute attribute);

	abstract Option getOption();

	abstract void setOption(Option option);

	abstract String getAttributeImage();

	abstract void setAttributeImage(String attributeImage);

	abstract String getOptionImage();

	abstract void setOptionImage(String optionImage);
}
