package com.softslate.commerce.businessobjects.payment;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.daos.payment.PaymentDAO;
import com.softslate.commerce.daos.payment.PaymentGatewayDAO;

/**
 * Processes business logic having to do with payments. Created in the Struts
 * layer in response to requests for viewing, editing, adding, and deleting
 * payments.
 * <p>
 * <code>BasicPaymentProcessor</code> is the default implementation of the
 * <code>PaymentProcessor</code> interface for the application.
 * 
 * @author David Tobey
 * 
 */
public class BasicAdminPaymentProcessor extends BaseBusinessProcessor implements
		AdminPaymentProcessor {

	static Log log = LogFactory.getLog(BasicPaymentProcessor.class);

	public Map loadPaymentsAndCount(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading payments and count");
		PaymentGatewayDAO gatewayDAO =  getDaoFactory()
				.createDAO(PaymentGatewayDAO.class);
		return gatewayDAO.loadPaymentsAndCount(parameters);
	}

	public Map addPayment(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Adding payment: " + code);
		}
		Map results = new HashMap();
		Payment payment = (Payment) getBusinessObjectFactory().createObject(
				"paymentImplementer");
		if (parameters.get("amount") != null
				&& parameters.get("amount").equals("")) {
			parameters.remove("amount");
		}
		BeanUtils.copyProperties(payment, parameters);
		payment.setCreated(formatDateTime(new java.util.Date()));
		Order order = (Order) getBusinessObjectFactory().createObject(
				"orderImplementer");
		order.setOrderID(payment.getOrderID());
		payment.setOrder(order);
		PaymentDAO dao = (PaymentDAO) getDaoFactory().createDAO(
				"paymentDAOImplementer");
		dao.setPayment(payment);
		dao.insertPayment(true);
		results.put("newRecord", dao.getPayment());
		return results;
	}

	public Payment loadPaymentFromID(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading payment from id");
		Payment payment = (Payment) getBusinessObjectFactory().createObject(
				"paymentImplementer");
		PaymentDAO dao = (PaymentDAO) getDaoFactory().createDAO(
				"paymentDAOImplementer");
		BeanUtils.copyProperties(payment, parameters);
		dao.setPayment(payment);
		dao.loadPaymentFromID();
		return dao.getPayment();
	}

	public Map editPayment(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Editing payment: " + code);
		}
		Map results = new HashMap();
		Payment payment = loadPaymentFromID(parameters);
		if (parameters.get("amount") != null
				&& parameters.get("amount").equals("")) {
			parameters.remove("amount");
		}
		BeanUtils.copyProperties(payment, parameters);
		Order order = (Order) getBusinessObjectFactory().createObject(
				"orderImplementer");
		order.setOrderID(payment.getOrderID());
		payment.setOrder(order);
		PaymentDAO dao = (PaymentDAO) getDaoFactory().createDAO(
				"paymentDAOImplementer");
		dao.setPayment(payment);
		dao.updatePayment(true);
		results.put("updatedRecord", dao.getPayment());
		return results;
	}

	public Map editDeletePayments(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Editing/deleting multiple payments");
		PaymentGatewayDAO dao =  getDaoFactory().createDAO(PaymentGatewayDAO.class);
		Map results = dao.editDeletePayments(parameters, getBusinessObjectFactory());
		return results;
	}

	public void deletePayment(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("paymentID");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Deleting payment: " + code);
		}
		Payment payment = (Payment) getBusinessObjectFactory().createObject(
				"paymentImplementer");
		BeanUtils.copyProperties(payment, parameters);
		PaymentDAO dao = (PaymentDAO) getDaoFactory().createDAO(
				"paymentDAOImplementer");
		dao.setPayment(payment);
		dao.deletePayment(true);
		return;
	}

	public Collection loadCardInfoStatistics() throws Exception {
		PaymentGatewayDAO dao = (PaymentGatewayDAO) getDaoFactory().createDAO(
				"paymentGatewayDAOImplementer");
		return dao.loadCardInfoStatistics();
	}
	
	public int purgeCardInfo(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Purging card information: " + parameters);
		PaymentGatewayDAO dao = (PaymentGatewayDAO) getDaoFactory().createDAO(
				"paymentGatewayDAOImplementer");
		return dao.purgeCardInfo(true, parameters);
	}
}
