package com.softslate.commerce.businessobjects.product;

import java.io.Serializable;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing a product within a category in the system.
 * <code>ProductCategoryBean</code> is the default implementation of the
 * <code>ProductCategory</code> interface for the application.
 * <p>
 * The database stores product categories in the <code>sscProductCategory</code>
 * table, and this interface can be used to represent a single row of that
 * table. The Hibernate configuration file
 * <code>ProductCategoryBean.hbm.xml</code> maps that table's columns to the
 * properties in this object.
 * 
 * @author David Tobey
 */
@Audited
@AuditTable("AudProductCategory")
public class ProductCategoryBean extends BaseBusinessObject implements
		ProductCategory, Serializable {

	private static final long serialVersionUID = 3258678291553548501L;

	static Log log = LogFactory.getLog(ProductCategoryBean.class);

	private int productCategoryID = 0;

	public int getProductCategoryID() {
		return (this.productCategoryID);
	}

	public void setProductCategoryID(int productCategoryID) {
		this.productCategoryID = productCategoryID;
	}

	private int productID = 0;

	public int getProductID() {
		return (this.productID);
	}

	public void setProductID(int productID) {
		this.productID = productID;
	}

	private int categoryID = 0;

	public int getCategoryID() {
		return (this.categoryID);
	}

	public void setCategoryID(int categoryID) {
		this.categoryID = categoryID;
	}

	private int productCategoryOrder = 0;

	public int getProductCategoryOrder() {
		return productCategoryOrder;
	}

	public void setProductCategoryOrder(int productCategoryOrder) {
		this.productCategoryOrder = productCategoryOrder;
	}

	private Product product;

	public Product getProduct() {
		return product;
	}

	public void setProduct(Product product) {
		this.product = product;
	}

	private Category category;

	public Category getCategory() {
		return category;
	}

	public void setCategory(Category category) {
		this.category = category;
	}
}
