package com.softslate.commerce.businessobjects.product;

import java.util.Collection;
import java.util.Locale;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface representing a list of products in the the system. There are three
 * types of <code>ProductList</code> that are determined by the value of the
 * <code>productListType</code> property:
 * <ol>
 * <li>The list of all the active products in the store.</li>
 * <li>The list of products under a given category.</li>
 * <li>The list of products matching a given search request.</li>
 * </ol>
 * <p>
 * An instance of this interface is created from the Struts layer in response to
 * requests to view the active products in the store, a given category's
 * products, or a the results of a search. The products become accessible
 * through the <code>productListCollection</code> property of this
 * <code>ProductList</code>.
 * <p>
 * When the application needs to create an instance that implements
 * <code>ProductList</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "productListImplementer" setting in the <code>appComponents.properties</code>
 * file.
 * <p>
 * The default "productListImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.ProductListBean}.
 * 
 * @author David Tobey
 * 
 */
public interface ProductList extends BusinessObject {
	abstract int getProductListType();

	abstract void setProductListType(int productListType);

	abstract int getCategoryID();

	abstract void setCategoryID(int categoryID);

	abstract int getManufacturerID();

	abstract void setManufacturerID(int manufacturerID);
	
	abstract int getCustomerWishListID();

	abstract void setCustomerWishListID(int customerWishListID);

	abstract int getItemsPerPage();

	abstract void setItemsPerPage(int itemsPerPage);

	abstract int getFirstRow();

	abstract void setFirstRow(int firstRow);

	abstract int getItemCount();

	abstract void setItemCount(int itemCount);

	abstract String getSearchString();

	abstract void setSearchString(String searchString);

	abstract Collection getProductListCollection();

	abstract void setProductListCollection(Collection productListCollection);

	abstract void addFormatting(Locale locale, String currencyCode);
}
