package com.softslate.commerce.businessobjects.shipping;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.daos.shipping.ShippingMethodDAO;
import com.softslate.commerce.daos.shipping.ShippingMethodGatewayDAO;

/**
 * Processes business logic having to do with shipping methods. Created in the
 * Struts layer in response to requests for viewing, editing, adding, and
 * deleting shipping methods.
 * <p>
 * <code>BasicShippingMethodProcessor</code> is the default implementation of
 * the <code>ShippingMethodProcessor</code> interface for the application.
 * 
 * @author David Tobey
 */
public class BasicShippingMethodProcessor extends BaseBusinessProcessor
        implements ShippingMethodProcessor {

    static Log log = LogFactory.getLog(BasicShippingMethodProcessor.class);

    public Map loadShippingMethodsAndCount(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Loading shipping methods and count");
        ShippingMethodGatewayDAO gatewayDAO =  getDaoFactory()
                .createDAO(ShippingMethodGatewayDAO.class);
        return gatewayDAO.loadShippingMethodsAndCount(parameters);
    }

    public Map addShippingMethod(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String code = (String) parameters.get("code");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Adding shipping method: " + code);
        }
		Map results = new HashMap();
        ShippingMethod shippingMethod =  getBusinessObjectFactory()
                .createObject(ShippingMethod.class);
        BeanUtils.copyProperties(shippingMethod, parameters);
        ShippingMethodDAO dao = (ShippingMethodDAO) getDaoFactory().createDAO(
                "shippingMethodDAOImplementer");
        dao.setShippingMethod(shippingMethod);
        dao.insertShippingMethod(true);
		results.put("newRecord", dao.getShippingMethod());
		return results;
    }

    public ShippingMethod loadShippingMethodFromID(Map parameters)
            throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Loading shipping method from id");
        ShippingMethod shippingMethod =  getBusinessObjectFactory()
                .createObject(ShippingMethod.class);
        ShippingMethodDAO dao = (ShippingMethodDAO) getDaoFactory().createDAO(
                "shippingMethodDAOImplementer");
        BeanUtils.copyProperties(shippingMethod, parameters);
        dao.setShippingMethod(shippingMethod);
        dao.loadShippingMethodFromID();
		return dao.getShippingMethod();
    }

    public ShippingMethod loadShippingMethodFromCode(Map parameters)
            throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Loading shipping method from code");
        ShippingMethod shippingMethod =  getBusinessObjectFactory()
                .createObject(ShippingMethod.class);
        ShippingMethodDAO dao = (ShippingMethodDAO) getDaoFactory().createDAO(
                "shippingMethodDAOImplementer");
        BeanUtils.copyProperties(shippingMethod, parameters);
        dao.setShippingMethod(shippingMethod);
        dao.loadShippingMethodFromCode();
        return dao.getShippingMethod();
    }

    public Map editShippingMethod(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String code = (String) parameters.get("code");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Editing shipping method: " + code);
        }
		Map results = new HashMap();
        ShippingMethod shippingMethod = null;
		if (parameters.get("shippingMethodID") != null) {
			shippingMethod = loadShippingMethodFromID(parameters);
		}
		if (shippingMethod == null && parameters.get("code") != null) {
			shippingMethod = loadShippingMethodFromCode(parameters);
		}
        BeanUtils.copyProperties(shippingMethod, parameters);
        ShippingMethodDAO dao = (ShippingMethodDAO) getDaoFactory().createDAO(
                "shippingMethodDAOImplementer");
        dao.setShippingMethod(shippingMethod);
        dao.updateShippingMethod(true);
		results.put("updatedRecord", dao.getShippingMethod());
		return results;
    }

	public Map editDeleteShippingMethods(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Editing/deleting multiple shipping methods");
		ShippingMethodGatewayDAO dao = (ShippingMethodGatewayDAO) getDaoFactory().createDAO(
				"shippingMethodGatewayDAOImplementer");
		Map results = dao.editDeleteShippingMethods(parameters, getBusinessObjectFactory());
		return results;
	}

    public void deleteShippingMethod(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String code = (String) parameters.get("code");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Deleting shipping method: " + code);
        }
        ShippingMethod shippingMethod =  getBusinessObjectFactory()
                .createObject(ShippingMethod.class);
        BeanUtils.copyProperties(shippingMethod, parameters);
        if (log.isDebugEnabled()) {
            log.debug("Is Active: " + shippingMethod.getIsActive());
        }
        ShippingMethodDAO dao = (ShippingMethodDAO) getDaoFactory().createDAO(
                "shippingMethodDAOImplementer");
        dao.setShippingMethod(shippingMethod);
        dao.deleteShippingMethod(true);
        return;
    }
}
