package com.softslate.commerce.businessobjects.shipping;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.daos.shipping.ShippingRateDAO;
import com.softslate.commerce.daos.shipping.ShippingRateGatewayDAO;

/**
 * Processes business logic having to do with shipping rates. Created in the
 * Struts layer in response to requests for viewing, editing, adding, and
 * deleting shipping rates.
 * <p>
 * <code>BasicShippingRateProcessor</code> is the default implementation of
 * the <code>ShippingRateProcessor</code> interface for the application.
 * 
 * @author David Tobey
 * 
 */
public class BasicShippingRateProcessor extends BaseBusinessProcessor implements
        ShippingRateProcessor {

    static Log log = LogFactory.getLog(BasicShippingRateProcessor.class);

    public Map loadShippingRatesAndCount(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Loading shipping rates and count");
        ShippingRateGatewayDAO gatewayDAO =  getDaoFactory()
                .createDAO(ShippingRateGatewayDAO.class);
        return gatewayDAO.loadShippingRatesAndCount(parameters);
    }

    public Map addShippingRate(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            log.info(getUser().getAdministrator().getUserName()
                    + ": Adding shipping rate");
        }
		Map results = new HashMap();
        ShippingRate shippingRate =  getBusinessObjectFactory()
                .createObject(ShippingRate.class);
        BeanUtils.copyProperties(shippingRate, parameters);
        ShippingMethod shippingMethod =  getBusinessObjectFactory()
                .createObject(ShippingMethod.class);
        shippingMethod.setShippingMethodID(shippingRate.getShippingMethodID());
        shippingRate.setShippingMethod(shippingMethod);
        ShippingRateDAO dao = (ShippingRateDAO) getDaoFactory().createDAO(
                "shippingRateDAOImplementer");
        dao.setShippingRate(shippingRate);
        dao.insertShippingRate(true);
		results.put("newRecord", dao.getShippingRate());
		return results;
    }

    public ShippingRate loadShippingRateFromID(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Loading shipping rate from id");
        ShippingRate shippingRate =  getBusinessObjectFactory()
                .createObject(ShippingRate.class);
        ShippingRateDAO dao = (ShippingRateDAO) getDaoFactory().createDAO(
                "shippingRateDAOImplementer");
        BeanUtils.copyProperties(shippingRate, parameters);
        dao.setShippingRate(shippingRate);
        dao.loadShippingRateFromID();
		return dao.getShippingRate();
    }

    public Map editShippingRate(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            log.info(getUser().getAdministrator().getUserName()
                    + ": Editing shipping rate");
        }
		Map results = new HashMap();
		ShippingRate shippingRate = loadShippingRateFromID(parameters);
        BeanUtils.copyProperties(shippingRate, parameters);
        ShippingMethod shippingMethod =  getBusinessObjectFactory()
                .createObject(ShippingMethod.class);
        shippingMethod.setShippingMethodID(shippingRate.getShippingMethodID());
        shippingRate.setShippingMethod(shippingMethod);
        ShippingRateDAO dao = (ShippingRateDAO) getDaoFactory().createDAO(
                "shippingRateDAOImplementer");
        dao.setShippingRate(shippingRate);
        dao.updateShippingRate(true);
		results.put("updatedRecord", dao.getShippingRate());
		return results;
    }

	public Map editDeleteShippingRates(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Editing/deleting multiple shipping rates");
		ShippingRateGatewayDAO dao = (ShippingRateGatewayDAO) getDaoFactory().createDAO(
				"shippingRateGatewayDAOImplementer");
		Map results = dao.editDeleteShippingRates(parameters, getBusinessObjectFactory());
		return results;
	}

    public void deleteShippingRate(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            log.info(getUser().getAdministrator().getUserName()
                    + ": Deleting shipping rate");
        }
        ShippingRate shippingRate =  getBusinessObjectFactory()
                .createObject(ShippingRate.class);
        BeanUtils.copyProperties(shippingRate, parameters);
        ShippingRateDAO dao = (ShippingRateDAO) getDaoFactory().createDAO(
                "shippingRateDAOImplementer");
        dao.setShippingRate(shippingRate);
        dao.deleteShippingRate(true);
        return;
    }
}
