package com.softslate.commerce.businessobjects.shipping;

import java.util.Collection;
import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods having to do with shipping rules. Created in
 * the Struts layer in response to requests for viewing, editing, adding, and
 * deleting shipping rules.
 * <p>
 * When the application needs to create an instance that implements
 * <code>ShippingRuleProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "shippingRuleProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "shippingRuleProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.shipping.BasicShippingRuleProcessor}.
 * 
 * @author Jason McWilliams
 */
public interface ShippingRuleProcessor extends BusinessProcessor {

	abstract Map loadShippingRulesAndCount(Map parameters) throws Exception;

	abstract Map addShippingRule(Map parameters) throws Exception;

	abstract ShippingRule loadShippingRuleFromID(Map parameters) throws Exception;

	abstract Map editShippingRule(Map parameters) throws Exception;

	abstract Map editDeleteShippingRules(Map parameters) throws Exception;

	abstract void deleteShippingRule(Map parameters) throws Exception;
	
	abstract Collection findAvailableShippingProcessors(Collection availableProcessors, int orderID) throws Exception;
	
	abstract Map processShippingRules(Collection newOrderItems) throws Exception;
	
	abstract Map processAddItems(Collection newOrderItems) throws Exception;
	
	abstract Map processEditItem(Collection newOrderItems) throws Exception;

	abstract Map processOnCheckout(Map parameters) throws Exception;
		
	abstract Map processOrderComplete(Map parameters) throws Exception;
	
}
