package com.softslate.commerce.customer.customer;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.customer.CustomerProcessor;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.payment.authorizeNetCIM.AuthorizeNetCIMPaymentProfileMin;
import com.softslate.commerce.businessobjects.payment.authorizeNetCIM.AuthorizeNetCIMProcessor;
import com.softslate.commerce.businessobjects.payment.authorizeNetCIM.AuthorizeNetCIMResult;
import com.softslate.commerce.businessobjects.payment.authorizeNetCIM.AuthorizeNetCIMResultCreateProfile;
import com.softslate.commerce.businessobjects.payment.authorizeNetCIM.AuthorizeNetCIMResultGetProfile;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts action class used for requests to manage a customer's CIM profiles.
 * Used for requests to "/AccountCIMManageProfiles.do".
 * 
 * @author Jason McWilliams
 * 
 */
public class AccountCIMManageProfilesAction extends BaseAction {
	static Log log = LogFactory.getLog(AccountCIMManageProfilesAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		BaseForm baseForm = (BaseForm) form;
		baseForm.setServlet(getServlet());
		baseForm.initializeProperties(mapping, request);
		synchronized (request.getSession()) {
			// Preserve the original request so the customer is redirected back
			// to the main addresses page after logging
			// in
			request.setAttribute("originalRequest", "/AccountCIMManageProfiles.do");

			// If the user is not logged in, it's an error.
			if (baseForm.getUser().getCustomer() == null) {
				return setErrorMessageAndForward(baseForm, mapping, "errors.userNotLoggedIn", "notLoggedIn");
			}

			// If this is not turned on, its an error
			String authorizeNetCIMManageProfiles = (String) baseForm.getSettings().getValue(
					"authorizeNetCIMManageProfiles");
			if (authorizeNetCIMManageProfiles == null || !authorizeNetCIMManageProfiles.equals("1")) {
				return setErrorMessageAndForward(baseForm, mapping, "errors.cimManageProfilesNotAllowed", "fatalError");
			}

			// If using the CIM API method, the only thing we do is show the
			// page as everything is done with ajax.
			String authorizeNetCIMHostedMethod = (String) baseForm.getSettings()
					.getValue("authorizeNetCIMHostedMethod");
			boolean useAPI = false;
			if ("API Only".equals(authorizeNetCIMHostedMethod)) {
				if (log.isInfoEnabled())
					log.info("Using API cim");
				useAPI = true;
				baseForm.getRequest().setAttribute("fromPage", "account");
			}

			// Start work
			Customer customer = baseForm.getUser().getCustomer();
			Order order = baseForm.getUser().getOrder();
			AuthorizeNetCIMProcessor cimProcessor =  baseForm.getBusinessObjectFactory()
					.createObject(AuthorizeNetCIMProcessor.class);
			cimProcessor.initialize();
			String internalErrorMessage = null;
			List<Map<String, String>> messages = null;
			String validProfileIDStr = null;
			long deleteID = 0;
			try {
				deleteID = Long.parseLong(request.getParameter("deleteID"));
			} catch (Exception ex) {
				deleteID = 0;
			}

			if (customer != null && customer.getCustomerID() > 0) {
				// Valid Customer
				long cimProfileID = 0;
				try {
					cimProfileID = Long.parseLong(customer.getCimProfileID());
				} catch (Exception ex) {
					cimProfileID = 0;
				}
				if (cimProfileID > 0) {
					// Valid profile exists ** we think
				} else if (cimProfileID < 0) {
					// Admin has set this as an invalid payment method for this
					// user
					return setErrorMessageAndForward(baseForm, mapping, "errors.cimManageProfilesCustomerNotAllowed",
							"nonFatalError");
				} else {
					// Need to create a profile
					AuthorizeNetCIMResultCreateProfile profile = null;
					try {
						profile = cimProcessor.createCustomerProfile(customer, null, order);
					} catch (Exception e) {
						if (log.isErrorEnabled()) {
							log.error(e);
						}
						return setErrorMessageAndForward(baseForm, mapping,
								"errors.cimManageProfilesCommunicationError", "nonFatalError");
					}
					if (profile.getError()) {
						// Throw Exception?
						internalErrorMessage = "Error creating new customer profile.";
						messages = profile.getMessages();
					} else if (profile.getProfileID() > 0) {
						CustomerProcessor customerProcessor =  baseForm.getBusinessObjectFactory()
								.createObject(CustomerProcessor.class);
						// Just update the customer record with the new
						// profileID
						Map parameters = new HashMap();
						parameters.put("cimProfileID", Long.toString(profile.getProfileID()));
						customerProcessor.updateCustomer(parameters);
						customer.setCimProfileID(Long.toString(profile.getProfileID()));
					} else {
						internalErrorMessage = "No Error, but returning profile id is 0??";
						messages = profile.getMessages();
					}
				}
				if (StringUtils.isNotBlank(internalErrorMessage) || (messages != null && messages.size() > 0)) {
					logProfileErrors(internalErrorMessage, messages, customer, "createCustomerProfile");
				}
				validProfileIDStr = customer.getCimProfileID();

			} else {
				// Something is wrong with customer, this is a bad error
				return setErrorMessageAndForward(baseForm, mapping, "errors.cimManageProfilesCustomerNotAllowed",
						"fatalError");
			}

			long validProfileID = 0;
			try {
				validProfileID = Long.parseLong(validProfileIDStr);
			} catch (Exception ex) {
				validProfileID = 0;
			}

			if (validProfileID > 0) {
				if (useAPI) {
					if (log.isInfoEnabled())
						log.info("Returning after validProfileID using API");
					return mapping.findForward("success");
				}

				Map<String, String> results = null;
				try {
					results = cimProcessor.getHostedProfilePage(validProfileID, "account");
				} catch (java.net.SocketException se) {
					if (log.isErrorEnabled()) {
						log.error(se);
					}
					return setErrorMessageAndForward(baseForm, mapping, "errors.cimManageProfilesCommunicationError",
							"nonFatalError");
				}
				if (results == null) {
					// No results, no good
					if (log.isInfoEnabled())
						log.info("Error getting hosted page: no results from gethostedProfilePage()");
					return setErrorMessageAndForward(baseForm, mapping, "errors.cimManageProfilesCommunicationError",
							"nonFatalError");
				} else {
					String xmlResultCode = results.get("xmlResultCode");
					String xmlMessageCode = results.get("xmlMessageCode");
					String xmlMessageText = results.get("xmlMessageText");
					String token = results.get("token");
					if ("Ok".equalsIgnoreCase(xmlResultCode) && StringUtils.isNotBlank(token)) {
						baseForm.getRequest().setAttribute("cimAccountToken", token);
						// Now also pull in the current payment methods
						// available to the user
						AuthorizeNetCIMResultGetProfile getProfileResult = null;
						try {
							getProfileResult = cimProcessor.getCustomerProfile(validProfileID);
						} catch (Exception e) {
							if (log.isErrorEnabled()) {
								log.error(e);
							}
							return setErrorMessageAndForward(baseForm, mapping,
									"errors.cimManageProfilesCommunicationError", "nonFatalError");
						}

						if (getProfileResult.getError()) {
							logProfileErrors("Error getting profile inforamtion.", getProfileResult.getMessages(),
									customer, "getCustomerProfile");
							return setErrorMessageAndForward(baseForm, mapping,
									"errors.cimManageProfilesGetProfileError", "nonFatalError");
						} else {
							if (getProfileResult.getProfileID() > 0
									&& getProfileResult.getProfileID() == validProfileID) {
								if (getProfileResult.getPaymentProfiles() != null
										&& getProfileResult.getPaymentProfiles().size() > 0) {
									Collection<AuthorizeNetCIMPaymentProfileMin> paymentProfiles = getProfileResult
											.getPaymentProfiles();
									String authorizeNetCIMAllowDeletePayments = (String) baseForm.getSettings()
											.getValue("authorizeNetCIMAllowDeletePayments");
									if (deleteID > 0 && "1".equals(authorizeNetCIMAllowDeletePayments)) {
										for (AuthorizeNetCIMPaymentProfileMin p : paymentProfiles) {
											if (p.getCustomerPaymentProfileId() == deleteID) {
												AuthorizeNetCIMResult deletePaymentResult = null;
												try {
													deletePaymentResult = cimProcessor.deleteCustomerPaymentProfile(
															validProfileID, deleteID);
												} catch (Exception ex) {
													// Just log this for now.
													logProfileErrors(
															"Exception deleting payment profile: " + ex.getMessage(),
															null, customer, "deleteCustomerPaymentProfile");
												}
												if (getProfileResult.getError()) {
													logProfileErrors("Error deleting payment profile",
															deletePaymentResult.getMessages(), customer,
															"deleteCustomerPaymentProfile");
												} else {
													// Dont reload everything
													// just remove it from the
													// collection so it can't be
													// seen for now and assume
													// it is really gone.
													paymentProfiles.remove(p);
												}
												break;
											}
										}
									}
									baseForm.getRequest().setAttribute("cimPaymentProfiles", paymentProfiles);
								}
							} else {
								// Not a good thing here
								logProfileErrors("Returned profileID does not match the sent profileID",
										getProfileResult.getMessages(), customer, "getCustomerProfile");
								return setErrorMessageAndForward(baseForm, mapping,
										"errors.cimManageProfilesProfileMismatch", "fatalError");

							}
						}
					} else {
						if (log.isInfoEnabled()) {
							log.info("Error getting hosted page - result code : " + xmlResultCode);
							log.info("Error getting hosted page - message code : " + xmlMessageCode);
							log.info("Error getting hosted page - message text : " + xmlMessageText);
							if (StringUtils.isBlank(token)) {
								log.info("Error getting hosted page - token is blank");
							}
						}
						return setErrorMessageAndForward(baseForm, mapping, "errors.cimManageProfilesGetProfileError",
								"nonFatalError");
					}
				}
			} else {
				// Give Up, Something is wrong
				return setErrorMessageAndForward(baseForm, mapping, "errors.cimManageProfilesUnableToCreateProfile",
						"fatalError");
			}

			return mapping.findForward("success");
		}
	}

	private ActionForward setErrorMessageAndForward(BaseForm baseForm, ActionMapping mapping, String code,
			String forward) {
		baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(code));
		saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
		if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
			log.info("Validation error(s): " + baseForm.getErrors().toString());
		return mapping.findForward(forward);
	}

	private void logProfileErrors(String internalErrorMessage, List<Map<String, String>> messages, Customer customer,
			String command) {
		if (log.isInfoEnabled()) {
			if (customer != null && customer.getCustomerID() > 0) {
				log.info("CIM error " + command + " - customer: " + customer.getCustomerID());
			}
			log.info(StringUtils.stripToEmpty(internalErrorMessage));
			if (messages != null) {
				for (Map<String, String> message : messages) {
					log.info(StringUtils.stripToEmpty(message.get("code")) + ": "
							+ StringUtils.stripToEmpty(message.get("text")));
				}
			}
		}
	}
}
