package com.softslate.commerce.customer.order;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

/**
 * 
 * @author Jason McWilliams
 */
public class AddressBookAddressForm extends AddressesForm {

	private static final long serialVersionUID = -8154435627046192105L;

	static Log log = LogFactory.getLog(AddressBookAddressForm.class);

	private String customerAddressID = null;
	
	private String useForBilling = "false";

	private boolean isBilling = false;

	public String getUseForBilling() {
		return useForBilling;
	}

	public void setUseForBilling(String useForBilling) {
		this.useForBilling = useForBilling;
	}

	public String getCustomerAddressID() {
		return customerAddressID;
	}

	public void setCustomerAddressID(String customerAddressID) {
		this.customerAddressID = customerAddressID;
	}

	public boolean getIsBilling() {
		return isBilling;
	}

	public void setIsBilling(boolean isBilling) {
		this.isBilling = isBilling;
	}


	// If using for address book, we only validate the delivery fields. We will
	// validate the billing fields if they choose to make it their default
	// billing address.
	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);
		if ("true".equals(getUseForBilling())) {
			copyBillingToDelivery(false);
			validateBillingFields();
		} else if (getIsBilling()) {
			validateBillingFields();
		} else {
			copyBillingToDelivery();
			validateDeliveryFields();
		}

		// Ensure that the form submission is coming from the site itself
		if (getCsrfToken() == null || getCsrfToken().length() == 0
				|| !getCsrfToken().equals(getRequest().getSession().getAttribute("csrfToken"))) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.csrfError"));
		}
		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}

	public void copyBillingToDelivery() {
		if (getDeliveryUseBilling()) {
			copyBillingToDelivery(true);
		}
	}

	public void copyBillingToDelivery(boolean fromBilling) {
	
		// Set up a Class for reflection
		Class classDefinition = this.getClass();
		Class[] argumentTypes = null;
		Object[] arguments = null;
		
		// If the 'deliver to billing address' box was checked, copy
		// the billing fields into the delivery fields.
		String billingPrompt = "";
		String deliveryPrompt = "";
		try {
			billingPrompt = (String) getSettings().getValue("billingPrompt");
			deliveryPrompt = (String) getSettings().getValue("deliveryPrompt");
		} catch (Exception e) {
			deliveryPrompt = "";
			billingPrompt = "";
		}
		String[] fromPromptArray;
		String toPrompt;
		String toMethod;
		String fromMethod;
		if (fromBilling) {
			toPrompt = deliveryPrompt;
			fromPromptArray = billingPrompt.split(",");
			toMethod = "setDelivery";
			fromMethod = "getBilling";
		} else {
			toPrompt = billingPrompt;
			fromPromptArray = deliveryPrompt.split(",");
			toMethod = "setBilling";
			fromMethod = "getDelivery";
		}

		// Loop through all of the to address fields we have on the screen
		for (int i = 0; i < fromPromptArray.length; i++) {
		
			// Set up a Class for reflection
			String thisFieldString = fromPromptArray[i];
			
			// Only copy in if we prompt for the same ones
			if (toPrompt.indexOf(thisFieldString) != -1) {
				String toMethodString = toMethod
						+ formUtils().capitalize(thisFieldString);
				String fromMethodString = fromMethod
						+ formUtils().capitalize(thisFieldString);
				if (log.isDebugEnabled())
					log.debug("Copying: " + fromMethodString + " to: "
							+ toMethodString);
				String thisToValue = null;
				try {
					Method method = classDefinition.getMethod(toMethodString,
							argumentTypes);
					thisToValue = (String) method.invoke(this, arguments);
					Class[] argumentTypes2 = { String.class };
					Object[] arguments2 = { thisToValue };
					Method method2 = classDefinition.getMethod(
							fromMethodString, argumentTypes2);
					method2.invoke(this, arguments2);
				} catch (Exception e) {
					log.error("Copy failed: " + e.toString(), e);
				}
			}
		}
		return;
	}
	
	public Map describeAddressType(String addressType){
		// Set up a Class for reflection
		Map results = new HashMap();
		Class classDefinition = this.getClass();
		Class[] argumentTypes = null;
		Object[] arguments = null;
		String addressPrompt = (String) getSettings().getValue(addressType + "Prompt");
		String[] promptArray = addressPrompt.split(",");
		// Loop through all of the to address fields we have on the screen
		for (int i = 0; i < promptArray.length; i++) {
			// Set up a Class for reflection
			String thisFieldString = promptArray[i];
			// Only copy in if we prompt for the same ones
			String methodBase = addressType + formUtils().capitalize(thisFieldString);
			String getMethodString = "get" + formUtils().capitalize(methodBase);
			try {
				Method method = classDefinition.getMethod(getMethodString, argumentTypes);
				String value = (String) method.invoke(this, arguments);
				results.put(thisFieldString, value);
			} catch (Exception e) {
				log.error("Copy failed: " + e.toString(), e);
			}
		}
		return results;
	}

	// Aliases for billing addresses

	public String getFirstName() {
		return getBillingFirstName();
	}

	public void setFirstName(String firstName) {
		setBillingFirstName(firstName);
	}

	public String getLastName() {
		return getBillingLastName();
	}

	public void setLastName(String lastName) {
		setBillingLastName(lastName);
	}

	public String getOrganization() {
		return getBillingOrganization();
	}

	public void setOrganization(String organization) {
		setBillingOrganization(organization);
	}

	public String getAddress1() {
		return getBillingAddress1();
	}

	public void setAddress1(String address1) {
		setBillingAddress1(address1);
	}

	public String getAddress2() {
		return getBillingAddress2();
	}

	public void setAddress2(String address2) {
		setBillingAddress2(address2);
	}

	public String getCity() {
		return getBillingCity();
	}

	public void setCity(String city) {
		setBillingCity(city);
	}

	public String getState() {
		return getBillingState();
	}

	public void setState(String state) {
		setBillingState(state);
	}

	public String getOther() {
		return getBillingOther();
	}

	public void setOther(String other) {
		setBillingOther(other);
	}

	public String getPostalCode() {
		return getBillingPostalCode();
	}

	public void setPostalCode(String postalCode) {
		setBillingPostalCode(postalCode);
	}

	public String getCountry() {
		return getBillingCountry();
	}

	public void setCountry(String country) {
		setBillingCountry(country);
	}

	public String getPhone1() {
		return getBillingPhone1();
	}

	public void setPhone1(String phone1) {
		setBillingPhone1(phone1);
	}

	public String getPhone2() {
		return getBillingPhone2();
	}

	public void setPhone2(String phone2) {
		setBillingPhone2(phone2);
	}

	public String getEmail1() {
		return getBillingEmail1();
	}

	public void setEmail1(String email1) {
		setBillingEmail1(email1);
	}

	public String getEmail2() {
		return getBillingEmail2();
	}

	public void setEmail2(String email2) {
		setBillingEmail2(email2);
	}

	public String getExtra1() {
		return getBillingExtra1();
	}

	public void setExtra1(String extra1) {
		setBillingExtra1(extra1);
	}

	public String getExtra2() {
		return getBillingExtra2();
	}

	public void setExtra2(String extra2) {
		setBillingExtra2(extra2);
	}

	public String getExtra3() {
		return getBillingExtra3();
	}

	public void setExtra3(String extra3) {
		setBillingExtra3(extra3);
	}

	public String getExtra4() {
		return getBillingExtra4();
	}

	public void setExtra4(String extra4) {
		setBillingExtra4(extra4);
	}

	public String getExtra5() {
		return getBillingExtra5();
	}

	public void setExtra5(String extra5) {
		setBillingExtra5(extra5);
	}

	public String getNotes() {
		return getBillingNotes();
	}

	public void setNotes(String notes) {
		setBillingNotes(notes);
	}


}
