package com.softslate.commerce.customer.order;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

/**
 * Struts form class for submissions to record address information and a
 * selected shipping option during checkout. Used for requests to
 * "/CheckoutComboShipping.do".
 * <p>
 * <code>CheckoutComboShippingForm</code> is a subclass of
 * {@link com.softslate.commerce.customer.order.CheckoutAddressesForm}, and it inherits
 * its billing and delivery address properties.
 * <p>
 * The <code>validate</code> method of this class runs the
 * <code>validate</code> method of <code>CheckoutAddressesForm</code> to make sure
 * all the required billing and delivery fields are present. It then checks to
 * make sure a shipping option was selected, if the "shippingRequired" setting
 * is set for the store.
 * 
 * @author David Tobey
 */
public class CheckoutComboShippingForm extends CheckoutAddressesForm {

	private static final long serialVersionUID = 3618414934455891248L;

	static Log log = LogFactory.getLog(CheckoutComboShippingForm.class);

	private boolean isSubmitted = false;

	public boolean isSubmitted() {
		return isSubmitted;
	}

	public void setIsSubmitted(boolean isSubmitted) {
		this.isSubmitted = isSubmitted;
	}

	/**
	 * The shipping method selected by the user during checkout. For the
	 * {@link com.softslate.commerce.businessobjects.shipping.BasicShippingProcessor},
	 * this field corresponds to the <code>code</code> property of the
	 * <code>sscShippingMethod</code> database table.
	 */
	private String shippingOption = null;

	public String getShippingOption() {
		return shippingOption;
	}

	public void setShippingOption(String string) {
		shippingOption = string;
	}

	/**
	 * If "true" this property indicates that if the form is being submitted by
	 * a logged-in customer, the customer's account information should be
	 * updated.
	 */
	private String updateCustomer = "true";

	public String getUpdateCustomer() {
		return updateCustomer;
	}

	public void setUpdateCustomer(String updateCustomer) {
		this.updateCustomer = updateCustomer;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");

		// Validate the address fields
		super.validate(mapping, request);

		// Ensure a shipping option was selected
		try {
			String shippingRequired = (String) getSettings().getValue(
					"shippingRequired");
			if (shippingRequired.equals("1")) {
				validateShippingOption();
			}
		} catch (Exception e) {
		}

		setIsSubmitted(true);

		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}

	/**
	 * Ensures a the <code>shippingOption</code> property has been set.
	 */
	public void validateShippingOption() {
		if (getShippingOption() == null || getShippingOption().length() == 0) {
			String message = getMessage("order",
					"checkoutAddresses.shippingOption", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}
	}
}
