package com.softslate.commerce.customer.payment;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

//import com.paypal.soap.api.AckCodeType;
//import com.paypal.soap.api.GetExpressCheckoutDetailsResponseType;
import com.softslate.commerce.businessobjects.payment.IPayPalExpressPaymentProcessor;
import com.softslate.commerce.customer.order.CheckoutAction;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * 
 * Action class to handle the redirected request from PayPal following a PayPal
 * Express Checkout session in which the user's address and payment information
 * are defined directly with PayPal.
 * 
 * When PayPal redirects the user back to the application, this class triggers
 * the second of the three API calls of PayPal Express,
 * GetExpressCheckoutDetails, to retrieve the details of the user's interaction
 * with PayPal Express on the PayPal website.
 * 
 * The <code>execute</code> method of this class invokes
 * {@link com.softslate.commerce.businessobjects.payment.PayPalExpressProcessor#processGetExpressCheckoutDetails(Map)}
 * , or whatever class is defined in the "activePaymentProcessors" setting that
 * corresponds to the paypalExpressForm.jsp template, to communicate with
 * PayPal.
 * 
 * @author David Tobey
 */
public class PayPalExpressReturnAction extends CheckoutAction {
	static Log log = LogFactory.getLog(PayPalExpressReturnAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");

		// Initialize the BaseForm
		BaseForm baseForm = new BaseForm();
		baseForm.initializeProperties(mapping, request);

		synchronized (baseForm.getRequest().getSession()) {

			// Look up the PayPal Processor's name
			PayPalExpressSetAction pesa = new PayPalExpressSetAction();
			pesa.setServlet(getServlet());
			String processorName = pesa.lookUpPayPalProcessorName();
			// Use the API to call SetExpressCheckout and get the user's token
			IPayPalExpressPaymentProcessor ppep = (IPayPalExpressPaymentProcessor) baseForm
					.getBusinessObjectFactory().createObjectFromClassName(
							processorName);
			Map results = ppep.processGetExpressCheckoutDetails(null);

			String resultCode = (String) results.get("resultCode");
			String resultMessages[] = (String[]) results.get("resultMessages");

			if (null == resultCode)
				resultCode = new String("-1");

			if (!resultCode.equals("0")) {
				if (null != resultMessages && resultMessages.length > 0)
					for (int i = 0; i < resultMessages.length; i++)
						baseForm.getErrors().add(
								ActionMessages.GLOBAL_MESSAGE,
								new ActionMessage("errors.detail",
										resultMessages[i]));
				else
					baseForm.getErrors().add(
							ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.detail", resultCode
									+ " : " + "Unknown PayPal Error"));

				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				return baseForm.getMapping().findForward("failure");
			} else {

				// If successful, force the declined login flag to be set
				baseForm.getRequest().getSession().setAttribute(
						"declinedLoginBeforeCheckout", "true");

				// If successful, run process checkout addresses. This will
				// update the returned address info in the database, run
				// inventory and discounting checks, and do taxes if shipping is
				// not required.
				ActionForward af = processCheckoutAddresses(baseForm, response);

				// If that was successful, store the PayPal processor in the
				// session as the selected payment processor.
				if (af.getName().startsWith("success")) {
					if (log.isDebugEnabled())
						log
								.debug("Storing payment form and processor name in session for confirm screen.");
					baseForm.getRequest().getSession().setAttribute(
							"validatedPaymentForm", new PayPalExpressForm());
					baseForm.getRequest().getSession().setAttribute(
							"processorName", processorName);

				}

				// Skip shipping and taxes if called for by a setting
				String paypalExpressRedoShipping = (String) baseForm
						.getSettings().getValue("paypalExpressRedoShipping");
				String shippingAndTaxesProcessed = (String) baseForm
						.getRequest().getSession().getAttribute(
								"shippingAndTaxesProcessed");
				if (af.getName().equals("success")
						&& paypalExpressRedoShipping.equals("0")
						&& shippingAndTaxesProcessed != null
						&& shippingAndTaxesProcessed.equals("true")) {
					baseForm.getRequest().getSession().setAttribute(
							"shippingAndTaxesProcessed", "true");
				}

				// Return the user to the force login screen if called for
				String forceLogin = (String) baseForm.getSettings().getValue(
						"forceLogin");
				if (forceLogin.equals("1")
						&& baseForm.getUser().getCustomer() == null) {
					if (log.isDebugEnabled())
						log.debug("Forwarding to the force login screen.");
					return baseForm.getMapping().findForward(
							"successForceLogin");
				}

				// Return the user straight to the confirmation screen if called
				// for
				if (af.getName().equals("success")
						&& paypalExpressRedoShipping.equals("0")
						&& shippingAndTaxesProcessed != null
						&& shippingAndTaxesProcessed.equals("true")) {
					return baseForm.getMapping().findForward(
							"successNoPaymentRequiredConfirm");
				}

				// Return the customer to the shipping selection screen
				return af;

			}
		}
	}
}
