package com.softslate.commerce.daos.order;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.order.OrderDelivery;
import com.softslate.commerce.businessobjects.order.OrderItem;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscOrderItem</code> table of the database.
 * 
 * @author David Tobey
 * 
 */
public class OrderItemDAOHibernate extends BaseDAO implements OrderItemDAO {

    static Log log = LogFactory.getLog(OrderItemDAOHibernate.class);

    private OrderItem orderItem = null;

    public OrderItem getOrderItem() {
        return (this.orderItem);
    }

    public void setOrderItem(OrderItem orderItem) {
        this.orderItem = orderItem;
    }

    public void loadOrderItemFromID() throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            OrderItem record = (OrderItem) session.get(OrderItem.class,
                    new Integer(getOrderItem().getOrderItemID()));
            PropertyUtils.copyProperties(getOrderItem(), record);
            setOrderItem(record);
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void insertOrderItem() throws DataAccessException {
        insertOrderItem(false);
    }

    public void insertOrderItem(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            OrderDelivery od = (OrderDelivery) session.load(
                    OrderDelivery.class, new Integer(getOrderItem()
                            .getOrderDeliveryID()));
            getOrderItem().setOrder(od.getOrder());
            session.save(getOrderItem());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void updateOrderItem() throws DataAccessException {
        updateOrderItem(false);
    }

    public void updateOrderItem(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            OrderDelivery od = (OrderDelivery) session.load(
                    OrderDelivery.class, new Integer(getOrderItem()
                            .getOrderDeliveryID()));
            getOrderItem().setOrder(od.getOrder());
            session.update(getOrderItem());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void deleteOrderItem() throws DataAccessException {
        deleteOrderItem(false);
    }

    public void deleteOrderItem(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
			setOrderItem((OrderItem) session.load(OrderItem.class, new Integer(
					getOrderItem().getOrderItemID())));
			session.delete(getOrderItem());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }
}
