package com.softslate.commerce.daos.payment;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.payment.Payment;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscPayment</code> table of the database.
 * 
 * @author David Tobey
 * 
 */
public class PaymentDAOHibernate extends BaseDAO implements DAOInterface,
		PaymentDAO {

	static Log log = LogFactory.getLog(PaymentDAOHibernate.class);

	private Payment payment = null;

	public Payment getPayment() {
		return (this.payment);
	}

	public void setPayment(Payment payment) {
		this.payment = payment;
	}

	public void loadPaymentFromID() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Payment record = (Payment) session.get(Payment.class, new Integer(
					getPayment().getPaymentID()));
			if (record != null) {
				if (record.getCreditCardNumber() != null) {
					record.setDecryptedNumber(getDAOUtils().getDecrypted(
							record.getCreditCardNumber(),
							(String) getSettings().getValue(
									"paymentEncryptionType"),
							getAppSettings().getProperty("twoWayKeyFile")));
				}
				PropertyUtils.copyProperties(getPayment(), record);
			}
			setPayment(record);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public String loadNumberByID() throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "payment.selectNumberFromID");
			q.setInteger("paymentID", getPayment().getPaymentID());
			String number = (String) q.uniqueResult();
			return number;
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public void insertPayment() throws DataAccessException {
		insertPayment(false);
	}

	public void insertPayment(boolean commit) throws DataAccessException {
		if (getPayment().getDecryptedNumber() != null) {
			getPayment().setCreditCardNumber(
					getDAOUtils().getEncrypted(
							getPayment().getDecryptedNumber(),
							(String) getSettings().getValue(
									"paymentEncryptionType"),
							getAppSettings().getProperty("twoWayKeyFile"),
							(String) getSettings().getValue("oneWayEncryptionAlgorithm")));
		}
		Session session = getDaoFactory().startSession();
		try {
			session.save(getPayment());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void updatePayment() throws DataAccessException {
		updatePayment(false);
	}

	public void updatePayment(boolean commit) throws DataAccessException {
		// Do not re-encrypt the password if the encryption type is one way.
		String encType = (String) getSettings().getValue(
				"paymentEncryptionType");
		if (encType != null
				&& !encType.equals("oneWay")
				&& getPayment().getDecryptedNumber() != null
				&& getPayment().getDecryptedNumber().length() > 0
				&& !getPayment().getDecryptedNumber().equals(
						"[oneWayEncryption]")
				&& !getPayment().getDecryptedNumber().equals(
						"[keyPairNoStoreEncryption]")) {
			getPayment().setCreditCardNumber(
					getDAOUtils().getEncrypted(
							getPayment().getDecryptedNumber(), encType,
							getAppSettings().getProperty("twoWayKeyFile"),
							(String) getSettings().getValue("oneWayEncryptionAlgorithm")));
		} else {
			// Use the password that's already in the database.
			try {
				String number = loadNumberByID();
				getPayment().setCreditCardNumber(number);
			} catch (Exception e) {
			}
		}
		Session session = getDaoFactory().startSession();
		try {
			session.update(getPayment());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void deletePayment() throws DataAccessException {
		deletePayment(false);
	}

	public void deletePayment(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			setPayment((Payment) session.load(Payment.class, new Integer(
					getPayment().getPaymentID())));
			session.delete(getPayment());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}
}
