package com.softslate.commerce.daos.product;

import com.softslate.commerce.businessobjects.product.ProductReview;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;

import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscProductReview</code> table of the
 * database.
 * 
 * @author Tony Zakula
 */

public class ProductReviewDAOHibernate extends BaseDAO implements
		ProductReviewDAO {

	static Log log = LogFactory.getLog(ProductReviewDAOHibernate.class);

	private ProductReview productReview = null;

	public ProductReview getProductReview() {
		return (this.productReview);
	}

	public void setProductReview(ProductReview productReview) {
		this.productReview = productReview;
	}

	public void loadReviewFromID() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			ProductReview pr = (ProductReview) session.get(ProductReview.class,
					getProductReview().getProductReviewID());
			PropertyUtils.copyProperties(getProductReview(), pr);
			setProductReview(pr);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void insertProductReview(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			getProductReview().setCreated(formatDateTime(new java.util.Date()));
			session.save(getProductReview());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
			evictCache();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void updateProductReview(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			session.update(getProductReview());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void deleteProductReview(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			setProductReview((ProductReview) session.load(ProductReview.class,
					getProductReview().getProductReviewID()));
			session.delete(getProductReview());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
			evictCache();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

}
