package com.softslate.commerce.daos.shipping;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.shipping.ShippingMethod;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;


/**
 * Generic DAO to manage the <code>sscShippingMethod</code> table of the
 * database.
 * 
 * @author David Tobey
 * 
 */
public class ShippingMethodDAOHibernate extends BaseDAO implements
        DAOInterface, ShippingMethodDAO {

    static Log log = LogFactory.getLog(ShippingMethodDAOHibernate.class);

    private ShippingMethod shippingMethod = null;

    public ShippingMethod getShippingMethod() {
        return (this.shippingMethod);
    }

    public void setShippingMethod(ShippingMethod shippingMethod) {
        this.shippingMethod = shippingMethod;
    }

    public void loadShippingMethodFromID() throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            ShippingMethod record = (ShippingMethod) session.get(
                    ShippingMethod.class, new Integer(getShippingMethod()
                            .getShippingMethodID()));
            PropertyUtils.copyProperties(getShippingMethod(), record);
            setShippingMethod(record);
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void loadShippingMethodFromCode() throws Exception {
        Session session = getDaoFactory().startSession();
        try {
			Query q = getNamedQuery(session, "shippingMethod.selectFromCode");
			q.setString("code", getShippingMethod().getCode());
			q.setCacheable(true);
			ShippingMethod loadedRecord= (ShippingMethod) q.uniqueResult();
			setShippingMethod(loadedRecord);
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void insertShippingMethod() throws DataAccessException {
        insertShippingMethod(false);
    }

    public void insertShippingMethod(boolean commit)
            throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.save(getShippingMethod());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void updateShippingMethod() throws DataAccessException {
        updateShippingMethod(false);
    }

    public void updateShippingMethod(boolean commit)
            throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.update(getShippingMethod());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void deleteShippingMethod() throws DataAccessException {
        deleteShippingMethod(false);
    }

    public void deleteShippingMethod(boolean commit)
            throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            setShippingMethod((ShippingMethod) session.load(
                    ShippingMethod.class, new Integer(getShippingMethod()
                            .getShippingMethodID())));
            session.delete(getShippingMethod());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }
}
